﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.DataAnalysis.Symbolic;
using HeuristicLab.Random;
using System.Collections.Generic;
using System.Linq;
using Variable = HeuristicLab.Problems.DataAnalysis.Symbolic.Variable;

namespace HeuristicLab.Benchmarks {
  public static class Util {
    public static Dataset CreateRandomDataset(IRandom random, int rows, int columns) {
      double[,] data = new double[rows, columns];
      for (int i = 0; i < rows; i++) {
        for (int j = 0; j < columns; j++) {
          data[i, j] = random.NextDouble() * 2.0 - 1.0;
        }
      }
      IEnumerable<string> variableNames = new string[] { "y" }.Concat(Enumerable.Range(0, columns - 1).Select(x => "x" + x.ToString()));
      Dataset ds = new Dataset(variableNames, data);
      return ds;
    }

    public static IEnumerable<int> IntRange(int start, int end, int step, bool closed = true) {
      return SequenceGenerator.GenerateSteps((decimal)start, end, step, closed).Select(x => (int)x);
    }

    public static ISymbolicExpressionTree[] CreateRandomTrees(IRandom twister, IDataset dataset, ISymbolicExpressionGrammar grammar, int popSize) {
      return CreateRandomTrees(twister, dataset, grammar, popSize, 1, 200, 3, 3);
    }

    public static ISymbolicExpressionTree[] CreateRandomTrees(IRandom twister, IDataset dataset, ISymbolicExpressionGrammar grammar,
      int popSize, int minSize, int maxSize,
      int maxFunctionDefinitions, int maxFunctionArguments) {
      foreach (Variable variableSymbol in grammar.Symbols.OfType<Variable>()) {
        variableSymbol.VariableNames = dataset.VariableNames;
      }
      ISymbolicExpressionTree[] randomTrees = new ISymbolicExpressionTree[popSize];
      for (int i = 0; i < randomTrees.Length; i++) {
        randomTrees[i] = ProbabilisticTreeCreator.Create(twister, grammar, maxSize, 10);
      }
      return randomTrees;
    }

    public static void InitTree(ISymbolicExpressionTree tree, MersenneTwister twister, List<string> varNames) {
      foreach (var node in tree.IterateNodesPostfix()) {
        if (node is VariableTreeNode) {
          var varNode = node as VariableTreeNode;
          varNode.Weight = twister.NextDouble() * 20.0 - 10.0;
          varNode.VariableName = varNames[twister.Next(varNames.Count)];
        } else if (node is ConstantTreeNode) {
          var constantNode = node as ConstantTreeNode;
          constantNode.Value = twister.NextDouble() * 20.0 - 10.0;
        }
      }
    }
  }
}
